<?php

/*
 * Copyright (C) 2021 AdumoOnline South Africa - All Rights Reserved
 *
 * Unauthorized copying, modifying or distribution of this file,
 * via any medium is strictly prohibited.
 *
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from AdumoOnline.
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

class AdumoOnline extends PaymentModule
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'adumoonline';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.1';
        $this->author = 'AdumoOnline';
        $this->controllers = array('payment', 'validation');
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('AdumoOnline');
        $this->description = $this->l('Process Payments through AdumoOnline\'s secure virtual payment gateway');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall the AdumoOnline Payment module?');

        $this->limited_countries = array('ZA');

        $this->limited_currencies = array(
            'BWP', //Botswana Pula
            'GHS', //Ghanaian Cedi
            'KES', //Kenyan Shilling
            'USD', //US Dollar
            'SCR', //Seychellois Rupee
            'TZS', //Tanzanian Shilling
            'UGX', //Ugandan Shilling
            'ZMW', //Zambian Kwacha
            'MZN', //Mozambican Metcal
            'NGN', //Nigerian Naira
            'GBP', //British Pound
            'EUR', //Euro
            'MUR', //Mauritian Rupee
            'ZAR', //South African Rand
        );

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }

    public function install()
    {
        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');

            return false;
        }

        Configuration::updateValue('ADUMOONLINE_LIVE_MODE', false);

        include(dirname(__FILE__).'/sql/install.php');

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('paymentReturn');
    }

    public function uninstall()
    {
        Configuration::deleteByName('ADUMOONLINE_LIVE_MODE');

        include(dirname(__FILE__).'/sql/uninstall.php');

        return parent::uninstall();
    }

    public function getContent()
    {
        if (((bool) Tools::isSubmit('submitAdumoOnlineModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitAdumoOnlineModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'ADUMOONLINE_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode? Set this to no for testing purposes'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'desc' => $this->l('URL to post the payment information to, this will change when going live'),
                        'name' => 'ADUMOONLINE_VIRTUAL_END_POINT',
                        'label' => $this->l('Post URL'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'desc' => $this->l('You can get your merchant id by logging into your payment console at https://console.mygateglobal.com/'),
                        'name' => 'ADUMOONLINE_MERCHANT_ID',
                        'label' => $this->l('Merchant ID'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'ADUMOONLINE_APPLICATION_UID',
                        'label' => $this->l('Application UID'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'ADUMOONLINE_JWT_SECRET_KEY',
                        'label' => $this->l('JWT Secret Key'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    protected function getConfigFormValues()
    {
        return array(
            'ADUMOONLINE_LIVE_MODE' => Configuration::get('ADUMOONLINE_LIVE_MODE', true),
            'ADUMOONLINE_VIRTUAL_END_POINT' => Configuration::get('ADUMOONLINE_VIRTUAL_END_POINT', null),
            'ADUMOONLINE_MERCHANT_ID' => Configuration::get('ADUMOONLINE_MERCHANT_ID', null),
            'ADUMOONLINE_APPLICATION_UID' => Configuration::get('ADUMOONLINE_APPLICATION_UID', null),
            'ADUMOONLINE_JWT_SECRET_KEY' => Configuration::get('ADUMOONLINE_JWT_SECRET_KEY', null),
        );
    }

    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    public function hookPaymentOptions()
    {
        if (!$this->active) {
            return [];
        }

        $adumoonlineOption = new PaymentOption();
        $adumoonlineOption->setModuleName($this->name)
            ->setCallToActionText(
            // @TODO: Get payment name from configuration
                $this->trans('Pay by Credit Card', array(), 'Modules.AdumoOnline.Shop')
            )
            ->setAction($this->context->link->getModuleLink($this->name, 'redirect', array(), true))
            ->setAdditionalInformation($this->fetch('module:adumoonline/views/templates/hook/info.tpl'));

        return [
            $adumoonlineOption,
        ];
    }

    public function hookPaymentReturn($params)
    {
        if ($this->active == false) {
            return;
        }

        /** @var Order $order */
        $order = $params['order'];

        if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR')) {
            $this->smarty->assign('status', 'ok');
        }

        $this->smarty->assign(array(
            'id_order' => $order->id,
            'reference' => $order->reference,
            'params' => $params,
            'total' => Tools::displayPrice($order->getTotalPaid(), null, false),
        ));

        return $this->display(__FILE__, 'views/templates/hook/confirmation.tpl');
    }
}
