<?php

/*
 * Copyright (C) 2021 AdumoOnline South Africa - All Rights Reserved
 *
 * Unauthorized copying, modifying or distribution of this file,
 * via any medium is strictly prohibited.
 *
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from AdumoOnline.
 */

include(dirname(__FILE__).'/../../php-jwt/jwt.php');

class AdumoOnlineRedirectModuleFrontController extends ModuleFrontController
{
    /**
     * Do whatever you have to before redirecting the customer on the website of your payment processor.
     */
    public function postProcess()
    {
        if (Tools::getValue('action') == 'error') {
            return $this->displayError('An error occurred while trying to redirect to the payment page');
        } else {
            $cart = Context::getContext()->cart;
            $customer = Context::getContext()->customer;
            $currency = Context::getContext()->currency;
            $address = new Address((int) $cart->id_address_invoice);

            //Build the JWT request token
            $MerchantID = Configuration::get('ADUMOONLINE_MERCHANT_ID');
            $ApplicationID = Configuration::get('ADUMOONLINE_APPLICATION_UID');
            $amount = $cart->getOrderTotal();
            $merchantReference = $cart->id;
            $SecretKey = Configuration::get('ADUMOONLINE_JWT_SECRET_KEY');
            $iss = 'Prestashop Plugin';
            $authToken = '';

            $payloadArray = array();
            $payloadArray['iss'] = $iss;
            $payloadArray['cuid'] = $MerchantID;
            $payloadArray['auid'] = $ApplicationID;
            $payloadArray['amount'] = $amount;
            $payloadArray['mref'] = $merchantReference;
            $payloadArray['jti'] = base64_encode(random_bytes (32));
            $payloadArray['iat'] = time() - 60;
            $payloadArray['exp'] = time() + 600;

            $authToken = JWT::encode($payloadArray, $SecretKey);

            $this->context->smarty->assign(array(
                'cartId' => $cart->id,
                'secureKey' => $customer->secure_key,
                'merchantId' => Configuration::get('ADUMOONLINE_MERCHANT_ID'),
                'virtualEndPoint' => Configuration::get('ADUMOONLINE_VIRTUAL_END_POINT'),
                'applicationUid' => Configuration::get('ADUMOONLINE_APPLICATION_UID'),
                'mode' => (int) Configuration::get('ADUMOONLINE_LIVE_MODE'),
                'total' => $cart->getOrderTotal(),
                'jwttoken' => $authToken,
                'currencyCode' => $currency->iso_code,
                'products' => $cart->getProducts(),
                'shipping' => number_format($cart->getTotalShippingCost(), 2),
                'discount' => number_format($cart->getDiscountSubtotalWithoutGifts(), 2),
                'customer' => trim($customer->firstname.' '.$customer->lastname),
                'address' => $address,
            ));

            $this->setTemplate('module:adumoonline/views/templates/front/redirect.tpl');
        }
    }

    protected function displayError($message, $description = false)
    {
        $this->context->smarty->assign('path', $this->context->link->getPageLink('order', null, null, 'step=3'));

        array_push($this->errors, $this->module->l($message), $description);

        $this->context->smarty->assign(['errorMessage' => $message, 'errorDescription' => $description]);

        $this->setTemplate('module:adumoonline/views/templates/front/error.tpl');
    }
}
